@@ -1,3 +1,6 @@
+#
+# $Header: /cvsroot/devicetool/Solaris-DeviceTree/lib/Solaris/DeviceTree/Filesystem/MinorNode.pm,v 1.7 2003/12/12 11:11:55 honkbude Exp $
+#
 
 package Solaris::DeviceTree::Filesystem::MinorNode;
 
@@ -28,12 +31,7 @@ require Exporter;
 our %EXPORT_TAGS = ( 'all' => [ qw( S_IFBLK S_IFCHR ) ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
-use base qw( Exporter );
-use vars qw( $VERSION @EXPORT );
-
-@EXPORT = qw();
-$VERSION = '0.01';
-#our @ISA = qw( Solaris::DeviceTree::Node );
+our $VERSION = do { my @r = (q$Revision: 1.7 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker
 
 =pod
 
@@ -43,10 +41,24 @@ Solaris::DeviceTree::Filesystem::MinorNode - Minor node of the Solaris device fi
 
 =head1 SYNOPSIS
 
+Construction:
+
   use Solaris::DeviceTree::Filesystem;
-  $tree = new Solaris::DeviceTree::Filesystem;
+  $tree = Solaris::DeviceTree::Filesystem->new;
   @disks = $tree->find_nodes( type => 'disk' );
-  @minor = @disks->minor_nodes;
+  @minor = $disk[0]->minor_nodes;
+
+Data access methods:
+
+  $name = $minor->name;
+  $devfs_path = $minor->devfs_path;
+  ($major_num,$minor_num) = $minor->devt;
+  if( !defined $minor->nodetype ) { print "I knew that!"; }
+  $spectype = $minor->spectype
+  if( $minor->is_raw_device ) { ... }
+  if( $minor->is_block_device ) { ... }
+  $treenode = $minor->node;
+  $slice = $minor->slice;
 
 
 =head1 DESCRIPTION
@@ -54,25 +66,23 @@ Solaris::DeviceTree::Filesystem::MinorNode - Minor node of the Solaris device fi
 This class implements a minor node for a device file in the Solaris
 filesystem devicetree.
 
-This is an internal class to C<Solaris::DeviceTree::Filesystem>. There should be
+This is an internal class to L<Solaris::DeviceTree::Filesystem>. There should be
 no need to generate instances of this class in an application explicitly.
-Instances are generated only from L<Solaris::DeviceTree::Filesystem::minor_nodes()>.
+Instances are generated only from L<Solaris::DeviceTree::Filesystem::minor_nodes>.
 
 
 =head1 METHODS
 
 The following methods are available:
 
-
-=head2 $minor = new Solaris::DeviceTree::Libdevinfo::MinorNode($filepath, $devtree_node);
-
-The constructor takes a string holding the absolute path to
-the device file and a backreference to the
-C<Solaris::DeviceTree::Filesystem> object which generates this
-instance.
-
 =cut
 
+# The constructor takes a string holding the absolute path to
+# the device file and a backreference to the
+# L<Solaris::DeviceTree::Filesystem> object which generates this
+# instance. The constructor is considered private and should not be
+# used.
+
 sub new {
   my ($pkg, $filepath, $node) = @_;
 
@@ -99,11 +109,11 @@ sub new {
 
 =pod
 
-=head2 $name = $minor->name;
+=head2 name
 
-Return the name of the minor node. This is used e.g. as suffix
-of the device filename. For disks this is something like 'a' or
-'a,raw'.
+This method returns the name of the minor node.
+This is used e.g. as suffix of the device filename.
+For disks this is something like 'a' or 'a,raw'.
 
 =cut
 
@@ -114,7 +124,7 @@ sub name {
 
 =pod
 
-=head2 $path = $minor->devfs_path;
+=head2 devfs_path
 
 Return the complete physical path including the minor node
 
@@ -127,10 +137,10 @@ sub devfs_path {
 
 =pod
 
-=head2 ($majnum,$minnum) = $minor->devt;
+=head2 devt
 
 Return the major and minor device number as a pair for the node.
-The major numbers should be the same for all minor nodes return
+The major numbers should be the same for all minor nodes returned
 by a L<Solaris::DeviceTree::Libdevinfo> node.
 
 =cut
@@ -142,11 +152,11 @@ sub devt {
 
 =pod
 
-=head2 $type = $minor->nodetype
+=head2 nodetype
 
 Return the nodetype of the minor node. Because we can't
 find that out by looking at the filesystem we always return
-'undef'.
+C<undef>.
 
 =cut
 
@@ -156,12 +166,12 @@ sub nodetype {
 
 =pod
 
-=head2 $spectype = $minor->spectype
+=head2 spectype
 
-Returns the type of the minor node. Returns the scalar values
+Returns the type of the minor node. The return values
   $S_IFCHR   for a raw device
   $S_IFBLK   for a block device
-Both scalars are exported by default.
+are possible. Both scalars are exported by default.
 
 =cut
 
@@ -178,38 +188,36 @@ sub spectype {
 
 =pod
 
-=head2 if( $minor->is_raw_device ) { ... }
+=head2 is_raw_device
 
-Returns true if the minor node is a raw device
+Returns true if the minor node is a raw device.
 
 =cut
 
 sub is_raw_device {
   my $this = shift;
-#print "SPR ",$this->{_stat}->[$STAT_MODE] & $S_IFMT, " - ", $S_IFCHR, "\n";
   return ($this->{_stat}->[$STAT_MODE] & $S_IFMT) == $S_IFCHR;
 }
 
 =pod
 
-=head2 if( $minor->is_block_device ) { ... }
+=head2 is_block_device
 
-Returns true if the minor node is a block device
+Returns true if the minor node is a block device.
 
 =cut
 
 sub is_block_device {
   my $this = shift;
-#print "SPB\n";
   return ($this->{_stat}->[$STAT_MODE] & $S_IFMT) == $S_IFBLK;
 }
 
 =pod
 
-=head2 $node = $minor->node;
+=head2 node
 
-Returns the associated Solaris::DevinfoTree node.
-One DevinfoTree node can (and usually does) have multiple minor nodes.
+Returns the associated L<Solaris::DevinfoTree::Filesystem> node.
+A treenode can (and usually does) have multiple minor nodes.
 
 =cut
 
@@ -220,7 +228,7 @@ sub node {
 
 =pod
 
-=head2 $slice = $minor->slice;
+=head2 slice
 
 Returns the slice number associated with this minor node.
 
